#include "at32f403a_407.h"
#include "analog_output.h"
#include "shift_reg.h"
#include "utility.h"
#include "FreeRTOS.h"
#include "task.h"
#include "settings_api.h"
#include "dac_transport.h"
#include "monitoring.h"
#include "mux.h"
#include <stdio.h>

#undef DBG
#define DBG if(0)

#if defined (MAO_4)


uint16_t ao_mode;   // режим работы аналоговый

uint16_t ao_power;  // слово состояния питания выходных каналов

static bool set_value_flag = false; // 

led_t ao_output_led[4] = {OUT_1_G, OUT_2_G, OUT_3_G, OUT_4_G};



//
void ao_init(void)
{
    // Зеленые LED сигнализируют (вкл/выкл) входов.
    ao_led_update();    
    
    // Сдвиговые регистры
    sh_init();
    
    // Внешний DAC
    dac_init();
    
    // Режир работы выходов
    ao_mode = settings.ao_mode_bits;
    
    ao_set_value_flag();
    
#if 0    
    // Режим измерения
    ai_set_meas_mode();
    
    // Питание внешних датчиков
    ai_ext_sens_power(settings.ext_sens_power); 
    
    // Внешинй ADC
    ai_adc_init();
    
    // Modbus параметры
    ai_init_modbus_params();
#endif    
}

// Управление LED для выходов
void ao_led_update(void)
{
    for (int i = 0; i < AO_NUMBER; i++) 
    {
        if (settings.ao[i].state)
            leds[ao_output_led[i]].state = LED_ON;
        else
            leds[ao_output_led[i]].state = LED_OFF;
    }
}

// Включение/выключение каналов
void ao_set(void)
{
    for (int i = 0; i < AO_NUMBER; i++)
    {
        settings.ao[i].state = (uint8_t)((settings.ao_state_bits >> i) & 1);

        if (settings.ao[i].state == 0) {
            leds[ao_output_led[i]].state = LED_OFF;
        }
        else {
            leds[ao_output_led[i]].state = LED_ON;
        }
    }
}

// Включение/выключение каналов (безопасный режим)
void ao_set_save(void)
{
    for (int i = 0; i < AO_NUMBER; i++)
    {
        settings.ao_save[i].state = (uint8_t)((settings.ao_state_save_bits >> i) & 1);
    }
}         
         
// Установить режим работы каналов
void ao_set_mode(void)
{
    for (int i = 0; i < AO_NUMBER; i++)
    {
        settings.ao[i].mode = (uint8_t)((settings.ao_mode_bits >> i) & 1);
    }
}

// Установить режим работы каналов (безопасный режим)
void ao_set_mode_save(void)
{
    for (int i = 0; i < AO_NUMBER; i++)
    {
        settings.ao_save[i].mode = (uint8_t)((settings.ao_mode_save_bits >> i) & 1);
    }
}

//
void ao_set_value_flag(void)
{
    set_value_flag = true;
}

//
void ao_set_values(ao_t *ao)
{
    for (int i = 0; i < AO_NUMBER; i++)
    {
        if (ao[i].state) {
            dac_test((DAC_CHANNEL)i, ao[i].value);
        }
    }
}

// 
void dac_task(void *params)
{
    for (;;)
    {   
        // 
        ao_power = sh_ao_mode(ao_mode);
        
        // Обновить значения на выходах
        if (set_value_flag)
        {
            set_value_flag = false;
            ao_set_values(&settings.ao[0]);
        }
        
        DBG printf("AO_Power: %u\r\n", ao_power);
        
        vTaskDelay(100);
#if 0        
        ai_alarm = sh_ai_mode(ai_mode);
        ai_processing();
        ai_leds_processing();
        vTaskDelay(10);
#endif        
    }
}

#endif